{-# LANGUAGE FlexibleContexts #-}

module Main where

import qualified Data.ByteString as B
import qualified Data.Text as T
import Data.Yaml (decodeEither')
import System.Environment (getArgs)
import Tahoe.Announcement (Announcements (..))
import qualified Tahoe.Directory as TD
import Text.Megaparsec (parse)

import Tahoe.Download (announcementToMutableStorageServer, downloadDirectory)

main :: IO ()
main = do
    [announcementPath, dirReadCap] <- getArgs
    -- Load server announcements
    announcementsBytes <- B.readFile announcementPath
    let Right (Announcements announcements) = decodeEither' announcementsBytes

    -- Accept & parse read capability
    case parse TD.pReadSDMF "<argv>" (T.pack dirReadCap) of
        Left e -> print $ "Failed to parse cap: " <> show e
        Right r -> go announcements r
  where
    go announcements cap = do
        -- Download & decode the shares
        result <- downloadDirectory announcements cap announcementToMutableStorageServer

        -- Show the result
        putStrLn "Your result:"
        either print print result
